# _*_ coding: utf-8 _*_
#
# hspylib v0.11.187
#
# Package: main.hspylib.core.enums
"""Package initialization."""

__all__ = [
  'charset', 
  'content_type', 
  'enumeration', 
  'http_code', 
  'http_method'
]
__version__ = '0.11.187'
