# _*_ coding: utf-8 _*_
#
# hspylib v0.11.187
#
# Package: main.hspylib.core.config
"""Package initialization."""

__all__ = [
  'app_config', 
  'parser_factory', 
  'properties'
]
__version__ = '0.11.187'
